<?php
/**
 * Main Plugin
 *
 * @package   dorzki\WooCommerce\MinMaxQuantity
 * @author    Dor Zuberi <admin@dorzki.io>
 * @copyright 2023 dorzki
 * @version   1.0.0
 */

namespace dorzki\WooCommerce\MinMaxQuantity;

use WC_Product;

// Block if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


/**
 * Class Plugin
 *
 * @package dorzki\WooCommerce\MinMaxQuantity
 */
class Plugin {
	/**
	 * Plugin constructor.
	 */
	public function __construct() {
		add_filter( 'woocommerce_product_data_tabs', [ $this, 'add_min_max_limit_tab' ] );

		add_action( 'woocommerce_product_data_panels', [ $this, 'register_min_max_limit_fields' ] );
		add_action( 'woocommerce_process_product_meta_simple', [ $this, 'save_min_max_values' ] );

		add_filter( 'woocommerce_quantity_input_min', [ $this, 'set_minimum_quantity_limit' ], 10, 2 );
		add_filter( 'woocommerce_quantity_input_max', [ $this, 'set_maximum_quantity_limit' ], 10, 2 );
		add_filter( 'woocommerce_add_to_cart_validation', [ $this, 'check_min_max_limit' ], 10, 3 );

		add_filter( 'woocommerce_quantity_input_args', [ $this, 'set_min_max_values_in_cart' ], 10, 2 );
		add_filter( 'woocommerce_update_cart_validation', [ $this, 'check_cart_min_max_limit' ], 10, 4 );
	}


	/**
	 * Add new product tab.
	 *
	 * @param array $tabs List of registered tabs
	 *
	 * @return array
	 */
	public function add_min_max_limit_tab( array $tabs ): array {
		$tabs['product_min_max'] = [
			'label'  => __( 'Min Max Quantity', 'dorzki-wc-min-max-quantity' ),
			'target' => 'min_max_options',
			'class'  => [ 'show_if_simple' ],
		];

		return $tabs;
	}

	/**
	 * Output new tab fields.
	 *
	 * @return void
	 */
	public function register_min_max_limit_fields(): void {
		echo '<div id="min_max_options" class="panel woocommerce_options_panel">';
		echo '	<div class="options_group">';

		woocommerce_wp_text_input(
			[
				'id'                => '_min_quantity',
				'label'             => __( 'Minimum Quantity', 'dorzki-wc-min-max-quantity' ),
				'desc_tip'          => 'true',
				'description'       => __( 'Minimum quantity to purchase, set 1 for no minimum limitation.', 'dorzki-wc-min-max-quantity' ),
				'type'              => 'number',
				'custom_attributes' => [
					'min'  => 1,
					'step' => 1,
				],
			]
		);

		woocommerce_wp_text_input(
			[
				'id'                => '_max_quantity',
				'label'             => __( 'Maximum Quantity', 'dorzki-wc-min-max-quantity' ),
				'desc_tip'          => 'true',
				'description'       => __( 'Maximum quantity to purchase, set 0 for no maximum limitation.', 'dorzki-wc-min-max-quantity' ),
				'type'              => 'number',
				'custom_attributes' => [
					'min'  => 0,
					'step' => 1,
				],
			]
		);

		echo '	</div>';
		echo '</div>';
	}

	/**
	 * Save new fields values.
	 *
	 * @param int $product_id Product id.
	 *
	 * @return void
	 */
	public function save_min_max_values( int $product_id ): void {
		$min_field_value = ( isset( $_POST['_min_quantity'] ) ) ? sanitize_text_field( wp_unslash( $_POST['_min_quantity'] ) ) : null;
		$max_field_value = ( isset( $_POST['_max_quantity'] ) ) ? sanitize_text_field( wp_unslash( $_POST['_max_quantity'] ) ) : null;

		update_post_meta( $product_id, '_min_quantity', $min_field_value );
		update_post_meta( $product_id, '_max_quantity', $max_field_value );
	}


	/**
	 * Set minimum product quantity.
	 *
	 * @param int        $min Minimum value
	 * @param WC_Product $product Product data
	 *
	 * @return int
	 */
	public function set_minimum_quantity_limit( int $min, WC_Product $product ): int {
		$min_qty = get_post_meta( $product->get_ID(), '_min_quantity', true );

		if ( $min_qty ) {
			$min = (int) $min_qty;
		}

		return $min;
	}

	/**
	 * Set maximum product quantity.
	 *
	 * @param int        $max Maximum value
	 * @param WC_Product $product Product data
	 *
	 * @return int
	 */
	public function set_maximum_quantity_limit( int $max, WC_Product $product ): int {
		$max_qty = get_post_meta( $product->get_ID(), '_max_quantity', true );

		if ( $max_qty ) {
			$max = (int) $max_qty;
		}

		return $max;
	}


	/**
	 * Check product minimum and maximum quantity limitations.
	 *
	 * @param bool $passed Has validation passed
	 * @param int  $product_id Product id
	 * @param int  $quantity Product quantity
	 *
	 * @return bool
	 */
	public function check_min_max_limit( bool $passed, int $product_id, int $quantity ): bool {
		$product = wc_get_product( $product_id );

		if ( $product->is_type( 'simple' ) ) {
			$min = get_post_meta( $product_id, '_min_quantity', true );
			$max = get_post_meta( $product_id, '_max_quantity', true );

			if ( $quantity < $min ) {
				$passed = false;

				/* translators: %s Minimum Quantity */
				wc_add_notice( sprintf( __( 'You need a quantity of at least %s to add the product to cart.', 'dorzki-wc-min-max-quantity' ), "<strong>{$min}</strong>" ), 'error' );
			} elseif ( ! empty( $max ) && $quantity > $max ) {
				$passed = false;

				/* translators: %s Maximum Quantity */
				wc_add_notice( sprintf( __( 'You can purchase a maximum of %s units of this product.', 'dorzki-wc-min-max-quantity' ), "<strong>{$max}</strong>" ), 'error' );
			}
		}

		return $passed;
	}


	/**
	 * Set product quantity limitation in cart.
	 *
	 * @param array      $args Item arguments
	 * @param WC_Product $product Product data
	 *
	 * @return array
	 */
	public function set_min_max_values_in_cart( array $args, WC_Product $product ): array {
		if ( $product->is_type( 'simple' ) ) {
			$min = get_post_meta( $product->get_ID(), '_min_quantity', true );
			$max = get_post_meta( $product->get_ID(), '_max_quantity', true );

			$args['min_value'] = $min;
			$args['max_value'] = ( ! empty( $max ) ) ? $max : $args['max_value'];
		}

		return $args;
	}

	/**
	 * Check product quantity limitations in cart.
	 *
	 * @param bool   $passed Has validation passed
	 * @param string $cart_item_key Cart item key
	 * @param array  $cart_item Cart item
	 * @param int    $quantity Item quantity
	 *
	 * @return bool
	 */
	public function check_cart_min_max_limit( bool $passed, string $cart_item_key, array $cart_item, int $quantity ): bool {
		if ( $cart_item['data']->is_type( 'simple' ) ) {
			$min = get_post_meta( $cart_item['data']->get_ID(), '_min_quantity', true );
			$max = get_post_meta( $cart_item['data']->get_ID(), '_max_quantity', true );

			if ( $quantity < $min ) {
				$passed = false;

				/* translators: %s Minimum Quantity */
				wc_add_notice( sprintf( __( 'You need a quantity of at least %s to add the product to cart.', 'dorzki-wc-min-max-quantity' ), "<strong>{$min}</strong>" ), 'error' );
			} elseif ( ! empty( $max ) && $quantity > $max ) {
				$passed = false;

				/* translators: %s Maximum Quantity */
				wc_add_notice( sprintf( __( 'You can purchase a maximum of %s units of this product.', 'dorzki-wc-min-max-quantity' ), "<strong>{$max}</strong>" ), 'error' );
			}
		}

		if ( ! $passed ) {
			add_filter( 'woocommerce_update_cart_action_cart_updated', '__return_false' );
		}

		return $passed;
	}
}


// Init class.
new Plugin();
